/*
** Market v1.0
**
** A generic WWW market script
**
** Copyright (c) 12.2.95 by Andreas Ley <ley@rz.uni-karlsruhe.de>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/715 with HP-UX A.09.05
** In this environment, neither lint -u nor gcc -c -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Version history
**
** Version 1.0 - 12.2.95
**	Initial version
*/

char copyright[] = "@(#)(c) Copyright 1995 by Andreas Ley (ley@rz.uni-karlsruhe.de)";
char sccsid[] = "@(#)market v1.0 - A generic WWW market script";


/* General definitions */

#define TZ	"MEZ-1MESZ"
#define	MASTER	"webmaster@rz.uni-karlsruhe.de"
#define REFERENCE	"<A HREF=\"/~Andreas.Ley/\"><EM>Andreas Ley</EM></A>"
#define	MAX_AREAS	100
#define	MAX_FIELDS	100
#define	MAX_ENTRIES	1000

/* File locations */

#define	ROOT	"/usr/local/etc/httpd/lib/market"
#define	F_TITLE	".title"
#define	F_FORM	".form"
#define	F_EXPIRE	".expire"
#define	FORM_EXPIRE	"Expire"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <dirent.h>
#include <errno.h>
/*
#include "market_english.h"
*/
#include "market_german.h"

#define	FALSE	0		/* This is the naked Truth */
#define	TRUE	1		/* and this is the Light */

extern char	*sys_errlist[];

/* Functions from util.c */

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
void unescape_url(char *url);
void plustospace(char *str);

/* Functions from wildmat.c */

extern int wildmat();

/* Global variables */

typedef struct {
	char	*type;
	char	*flags;
	char	*title;
	char	*param;
	char	*value;
} field_t;

typedef struct {
	char	*id;
	char	**field;
} entry_t;

typedef struct {
	char	*dir;
	char	*title;
} area_t;

char	master[]=MASTER;
char	*script_name;
field_t	field[MAX_FIELDS];
int	field_max,field_expire;
entry_t	entry[MAX_ENTRIES];
int	entry_max;


int areacmp(const void *s1, const void *s2)
{
	return(strcmp(((area_t *)s1)->title,((area_t *)s2)->title));
}


int entrycmp(const void *s1, const void *s2)
{
	return(strcmp(((entry_t *)s1)->id,((entry_t *)s2)->id));
}


/* Prints header of a HTML-Request */
void show_header(char *subtitle, char *title, char *description)
{
	(void)printf("Content-type: text/html\n\n");
	(void)printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n");
	(void)printf("<HTML>\n");
	(void)printf("<HEAD>\n");
	(void)printf("<TITLE>%s%s%s%s%s</TITLE>\n",TITLE,subtitle?" - ":"",subtitle?subtitle:"",title?" - ":"",title?title:"");
	(void)printf("<LINK REV=\"made\" HREF=\"mailto:%s\">\n",MASTER);
	(void)printf("</HEAD>\n");
	(void)printf("<BODY>\n");
#ifdef SUBTITLE
	if (title)
		(void)printf("<EM>%s%s%s</EM><HR>\n",TITLE,subtitle?" - ":"",subtitle?subtitle:"");
#endif /* SUBTITLE */
	(void)printf("<H1>%s</H1>\n",title?title:TITLE);
	if (description) {
		(void)printf(description,master);
		(void)printf("<P>\n");
	}
}



/* Prints trailer of a HTML-Request */
void show_trailer()
{
	(void)printf(DISCLAIMER,master);
	(void)printf("\n%s\n",REFERENCE);
	(void)printf("</BODY>\n");
	(void)printf("</HTML>\n");
}



/* Shows an error page, based on wrong user input */
void show_error(char *error)
{
	show_header(NULL,ERROR,error);
	show_trailer();
	exit(0);
}



/* Shows an error page, based on internal misfunction */
void show_fatal(char *error)
{
	show_header(NULL,ERROR,error);
	(void)printf(ERROR_FATAL,master);
	(void)printf("<P>\n");
	show_trailer();
	exit(0);
}



/* Returns a Location: header */
void show_location(char *location)
{
	(void)printf("Location: %s\n",location);
	(void)printf("URI: %s\n\n",location);
}


char *make_id()
{
	static char	letter[]="abcdefghijklmnopqrstuvwxyz",vowel[]="aeiouy";
	static char	id[]="1234";
	struct timeval	tp;
	struct timezone	tzp;

	(void)gettimeofday(&tp,&tzp);
	srand((unsigned int)tp.tv_usec);
	id[0]=letter[rand()%strlen(letter)];
	do {
		id[1]=letter[rand()%strlen(letter)];
	} while((strchr(vowel,id[0])!=NULL)==(strchr(vowel,id[1])!=NULL));
	do {
		id[2]=letter[rand()%strlen(letter)];
	} while((strchr(vowel,id[1])!=NULL)&&(strchr(vowel,id[2])!=NULL));
	do {
		id[3]=letter[rand()%strlen(letter)];
	} while((strchr(vowel,id[2])!=NULL)==(strchr(vowel,id[3])!=NULL));
	return(id);
}



char *secure(char *text)
{
	char	*src,*dest;

	if (text) {
		for (src=text,dest=text;*src;src++)
			if ((*src&0x7f)>=0x20)
				*dest++=*src;
		*dest='\0';
	}
	return(text);
}



char *strip(char *line)
{
	char	*ptr;

	ptr=strchr(line,'\0');
	if (*(ptr-1)=='\n')
		ptr--;
	if (*(ptr-1)=='\r')
		ptr--;
	while (ptr>line && *(ptr-1)==' ')
		ptr--;
	*ptr='\0';
	return(line);
}



char *read_title(char *area)
{
	static char	buffer[8*1024];
	FILE	*src;

	if (!(src=fopen(F_TITLE,"r"))) {
		(void)sprintf(buffer,ERROR_READ_TITLE,area);
		show_fatal(buffer);
	}
	(void)fgets(buffer,sizeof(buffer),src);
	(void)fclose(src);
	return(strip(buffer));
}



int read_expire(char *area)
{
	static char	buffer[8*1024];
	FILE	*src;

	if (!(src=fopen(F_EXPIRE,"r"))) {
		(void)sprintf(buffer,ERROR_READ_EXPIRE,area);
		show_fatal(buffer);
	}
	(void)fgets(buffer,sizeof(buffer),src);
	(void)fclose(src);
	return(atoi(buffer));
}



void read_form(char *area)
{
	char		buffer[8*1024],*ptr,*nptr;
	FILE		*src;
	int			line=0;

	if (!(src=fopen(F_FORM,"r"))) {
		(void)sprintf(buffer,ERROR_READ_FORM,area);
		show_fatal(buffer);
	}

	field_max=0;
	while (fgets(buffer,sizeof(buffer),src)) {
		line++;
		(void)strip(buffer);
		if (*buffer=='\0'||*buffer=='#')
			continue;
		if (field_max>=MAX_FIELDS)
			show_fatal(ERROR_FIELDS);
		if (!(ptr=strchr(buffer,':'))) {
			(void)sprintf(buffer,ERROR_FORM,area,line);
			show_fatal(buffer);
		}
		*ptr++='\0';
		field[field_max].type=strdup(buffer);
		if (!(nptr=strchr(ptr,':'))) {
			(void)sprintf(buffer,ERROR_FORM,area,line);
			show_fatal(buffer);
		}
		*nptr++='\0';
		field[field_max].flags=strdup(ptr);
		ptr=nptr;
		if (!(nptr=strchr(ptr,':'))) {
			(void)sprintf(buffer,ERROR_FORM,area,line);
			show_fatal(buffer);
		}
		*nptr++='\0';
		field[field_max].title=strdup(ptr);
		field[field_max].param=strdup(nptr);
		field[field_max].value=NULL;
		field_max++;
	}
	(void)fclose(src);
}



void build_form()
{
	char	*ptr,*nptr;
	int	cnt;

	for (cnt=0;cnt<field_max;cnt++) {
		if (cnt)
			(void)printf("<BR>");
		(void)printf("<STRONG>%s:</STRONG>\n",field[cnt].title);
		if (!strcmp(field[cnt].type,"radio")) {
			ptr=field[cnt].param;
			while ((nptr=strchr(ptr,':'))) {
				*nptr++='\0';
				(void)printf("<INPUT TYPE=radio NAME=\"%s\" VALUE=\"%s\">%s",field[cnt].title,ptr,ptr);
				ptr=nptr;
			}
			(void)printf("<INPUT TYPE=radio NAME=\"%s\" VALUE=\"%s\">%s\n",field[cnt].title,ptr,ptr);
		}
		else if (!strcmp(field[cnt].type,"text")) {
			(void)printf("<BR><INPUT TYPE=text NAME=\"%s\" SIZE=%s>\n",field[cnt].title,field[cnt].param);
		}
	}
}



void parse_form()
{
	char	buffer[8*1024],*value,*title;
	int	cnt,cl;

	field_expire=0;
	cl=atoi(getenv("CONTENT_LENGTH"));
	if (cl==0)
		show_fatal(ERROR_CONTENT_LENGTH);
	while (cl&&(!feof(stdin))) {
		value=fmakeword(stdin,'&',&cl);
		plustospace(value);
		unescape_url(value);
		title=makeword(value,'=');
		if (!strcmp(FORM_EXPIRE,title))
			field_expire=atoi(value);
		else {
			for (cnt=0;cnt<field_max&&strcmp(field[cnt].title,title);cnt++);
			if (cnt>=field_max) {
				(void)sprintf(buffer,ERROR_FORM_TAG,title);
				show_fatal(buffer);
			}
			field[cnt].value=value;
		}
	}
}



void check_form(int expire)
{
	char	buffer[8*1024];
	int	cnt;

	if (field_expire==0)
		field_expire=expire;
	else if (field_expire>expire) {
		(void)sprintf(buffer,ERROR_FORM_EXPIRE,expire);
		show_error(buffer);
	}
	for (cnt=0;cnt<field_max;cnt++) {
		if (strchr(field[cnt].flags,'r')&&!(field[cnt].value&&*field[cnt].value)) {
			(void)sprintf(buffer,ERROR_REQUIRED,field[cnt].title);
			show_error(buffer);
		}
	}
}



int match_form()
{
	int	cnt;

	for (cnt=0;cnt<field_max;cnt++) {
		if (!wildmat(entry[entry_max].field[cnt],field[cnt].value))
			return(FALSE);
	}
	return(TRUE);
}



char *parse_id()
{
	char	buffer[8*1024],*value,*title,*id;
	int	cl;

	cl=atoi(getenv("CONTENT_LENGTH"));
	if (cl==0)
		show_fatal(ERROR_CONTENT_LENGTH);
	while (cl&&(!feof(stdin))) {
		value=fmakeword(stdin,'&',&cl);
		plustospace(value);
		unescape_url(value);
		title=makeword(value,'=');
		if (!strcmp("ID",title))
			id=strdup(value);
		else {
				(void)sprintf(buffer,ERROR_FORM_TAG,title);
				show_fatal(buffer);
		}
	}
	return(id);
}



void list_entries()
{
	int	cnt,fcnt;

	for (cnt=0;cnt<entry_max;cnt++) {
		if (cnt)
			(void)printf("<HR>\n");
		(void)printf("<DL COMPACT>\n");
		for (fcnt=0;fcnt<field_max;fcnt++) {
			(void)printf("<DT><STRONG>%s:</STRONG>\n",field[fcnt].title);
			if (strchr(entry[cnt].field[fcnt],'\n'))
				(void)printf("<DD>");
			(void)printf("%s\n",entry[cnt].field[fcnt]);
		}
		(void)printf("</DL>\n");
	}
}


char *put_entry(char *area)
{
	char		buffer[8*1024],*id,*ptr,*nptr;
	FILE		*dest;
	int		cnt;
	time_t		tloc;
	struct tm	*timeptr;

	do {
		id=make_id();
	} while (!access(id,F_OK));

	(void)umask(S_IWGRP|S_IWOTH);
	if ((dest=fopen(id,"w"))) {
		if ((ptr=getenv("REMOTE_ADDR"))&&*ptr)
			(void)fprintf(dest,"Addr: %s\n",ptr);
		if ((ptr=getenv("REMOTE_HOST"))&&*ptr)
			(void)fprintf(dest,"Host: %s\n",ptr);
		if ((ptr=getenv("REMOTE_IDENT"))&&*ptr)
			(void)fprintf(dest,"Ident: %s\n",ptr);
		if ((ptr=getenv("REMOTE_USER"))&&*ptr)
			(void)fprintf(dest,"User: %s\n",ptr);
		(void)time(&tloc);
		timeptr=localtime(&tloc);
		(void)strftime(buffer,sizeof(buffer),"%d %b %Y %X %Z",timeptr);
		(void)fprintf(dest,"Date: %s\n",buffer);
		timeptr->tm_mday+=field_expire;
		tloc=mktime(timeptr);
		timeptr=localtime(&tloc);
		(void)strftime(buffer,sizeof(buffer),"%d %b %Y %X %Z",timeptr);
		(void)fprintf(dest,"Expires: %s\n",buffer);
		/*
		(void)fprintf(dest,"Expires: %04d%02d%02d\n",timeptr->tm_year+1900,timeptr->tm_mon+1,timeptr->tm_mday);
		*/
		(void)fprintf(dest,"\n");
		for (cnt=0;cnt<field_max;cnt++) {
			if (field[cnt].value&&*field[cnt].value)
				if (strchr(field[cnt].value,'\n')) {
					(void)fprintf(dest,"%s:\n",field[cnt].title);
					ptr=field[cnt].value;
					while ((nptr=strchr(ptr,'\n'))) {
						*nptr='\0';
						if (*ptr=='.')
							(void)fprintf(dest,".");
						(void)fprintf(dest,"%s\n",secure(ptr));
						*nptr++='\n';
						ptr=nptr;
					}
					(void)fprintf(dest,"%s\n",secure(ptr));
					(void)fprintf(dest,".\n");
				}
				else
					(void)fprintf(dest,"%s: %s\n",field[cnt].title,secure(field[cnt].value));
		}
		(void)fclose(dest);
	}
	else {
		(void)sprintf(buffer,ERROR_WRITE,errno,sys_errlist[errno],area);
		show_fatal(buffer);
	}

	return(id);
}



void get_entry(char *area, char *id)
{
	char	buffer[8*1024],error[8*1024],*ptr;
	FILE	*src;
	int	cnt;

	if (!(src=fopen(id,"r"))) {
		(void)sprintf(error,ERROR_READ_ENTRY,id,area);
		show_fatal(error);
	}
	if (entry_max>=MAX_ENTRIES)
		show_fatal(ERROR_ENTRIES);
	entry[entry_max].id=strdup(id);
	entry[entry_max].field=(char **)calloc((size_t)field_max,sizeof(char *));
	while (fgets(buffer,sizeof(buffer),src)&&*strip(buffer));
	if (feof(src)) {
		(void)sprintf(error,ERROR_ENTRY_HEADER,id,area);
		show_fatal(error);
	}
	while (fgets(buffer,sizeof(buffer),src)) {
		(void)strip(buffer);
		ptr=strchr(buffer,':');
		if (!ptr) {
			(void)sprintf(error,ERROR_ENTRY_BODY_KEYWORD,id,area,buffer);
			show_fatal(error);
		}
		*ptr++='\0';
		while (*ptr==' ')
			ptr++;
		for (cnt=0;cnt<field_max&&strcmp(field[cnt].title,buffer);cnt++);
		if (cnt>=field_max) {
			(void)sprintf(error,ERROR_ENTRY_TAG,id,area,buffer);
			show_fatal(error);
		}
		if (*ptr)
			entry[entry_max].field[cnt]=strdup(ptr);
		else {
			if (!fgets(buffer,sizeof(buffer),src)) {
				(void)sprintf(error,ERROR_ENTRY_BODY_FIRST,id,area);
				show_fatal(error);
			}
			entry[entry_max].field[cnt]=strdup(strip(buffer));
			while(TRUE) {
				if (!fgets(buffer,sizeof(buffer),src)) {
					(void)sprintf(error,ERROR_ENTRY_BODY_MULTI,id,area);
					show_fatal(error);
				}
				if (!strcmp(strip(buffer),"."))
					break;
				entry[entry_max].field[cnt]=realloc((void *)entry[entry_max].field[cnt],strlen(entry[entry_max].field[cnt])+strlen(buffer)+2);
				(void)strcat(entry[entry_max].field[cnt],"\n");
				(void)strcat(entry[entry_max].field[cnt],buffer);
			}
		}
	}
	(void)fclose(src);
}



void list_area(char *area)
{
	DIR		*dir;
	struct dirent	*dp;

	entry_max=0;
	read_form(area);
	if (!(dir=opendir(".")))
		show_fatal(ERROR_DIRECTORY);
	while ((dp=readdir(dir)))
		if (*dp->d_name!='.') {
				get_entry(area,dp->d_name);
				entry_max++;
		}
	(void)closedir(dir);
	if (entry_max)
		qsort((void *)entry,(size_t)entry_max,sizeof(entry_t),entrycmp);

	show_header(NULL,read_title(area),LIST_DESC);
	if (entry_max)
		list_entries();
	else
		(void)printf(ERROR_EMPTY);
	show_trailer();
}



void search_area(char *area)
{
	char		buffer[8*1024],*ptr;
	DIR		*dir;
	struct dirent	*dp;
	int	cnt;

	read_form(area);
	if (strcmp(getenv("REQUEST_METHOD"),"POST")) {
		show_header(read_title(area),SEARCH_HEADER,SEARCH_DESC);
		(void)sprintf(buffer,"%s/search/%s",script_name,area);
		(void)printf(SEARCH_FORM_HEADER,buffer);
		build_form();
		(void)printf(SEARCH_FORM_TRAILER);
		show_trailer();
	}
	else {
		parse_form();
		for (cnt=0;cnt<field_max;cnt++) {
			for(ptr=field[cnt].value;*ptr;ptr++)
				if (*ptr==' '||*ptr=='\t'||*ptr=='\n')
					*ptr='*';
			ptr=field[cnt].value;
			field[cnt].value=malloc(strlen(ptr)+3);
			field[cnt].value[0]='*';
			(void)strcpy(field[cnt].value+1,ptr);
			field[cnt].value[strlen(ptr)+1]='*';
			field[cnt].value[strlen(ptr)+2]='\0';
			free((void *)ptr);
		}
		entry_max=0;
		if (!(dir=opendir(".")))
			show_fatal(ERROR_DIRECTORY);
		while ((dp=readdir(dir)))
			if (*dp->d_name!='.') {
					get_entry(area,dp->d_name);
					if (match_form())
						entry_max++;
			}
		(void)closedir(dir);
		if (entry_max)
			qsort((void *)entry,(size_t)entry_max,sizeof(entry_t),entrycmp);

		show_header(NULL,read_title(area),SEARCH_PUT_DESC);
		if (entry_max)
			list_entries();
		else
			(void)printf(SEARCH_EMPTY);
		show_trailer();
	}
}



void post_area(char *area)
{
	char		buffer[8*1024],*id;
	int		expire;

	read_form(area);
	expire=read_expire(area);
	if (strcmp(getenv("REQUEST_METHOD"),"POST")) {
		show_header(read_title(area),POST_HEADER,POST_DESC);
		(void)sprintf(buffer,"%s/post/%s",script_name,area);
		(void)printf(POST_FORM_HEADER,buffer);
		build_form();
		(void)printf(POST_FORM_EXPIRE,FORM_EXPIRE,expire);
		(void)printf(POST_FORM_TRAILER);
		show_trailer();
	}
	else {
		parse_form();
		check_form(expire);
		id=put_entry(area);
		show_header(read_title(area),POST_PUT_HEADER,POST_PUT_DESC);
		(void)printf(POST_PUT_SUCCESS,id);
		show_trailer();
	}
}



void delete_area(char *area)
{
	char		buffer[8*1024],*id;

	if (strcmp(getenv("REQUEST_METHOD"),"POST")) {
		show_header(read_title(area),DELETE_HEADER,DELETE_DESC);
		(void)sprintf(buffer,"%s/delete/%s",script_name,area);
		(void)printf(DELETE_FORM,buffer);
		show_trailer();
	}
	else {
		id=parse_id();
		if (!access(id,F_OK)) {
			if (!remove(id)) {
				show_header(read_title(area),DELETE_HEADER,DELETE_SUCCESS);
				show_trailer();
			}
			else {
				(void)sprintf(buffer,ERROR_DELETE,errno,sys_errlist[errno],area);
				show_fatal(buffer);
			}
		}
		else
			show_error(DELETE_NEXIST);
	}
}



void select_area(char *area)
{
	show_header(NULL,read_title(area),SELECT_DESC);
	(void)printf("<BLOCKQUOTE>\n");
	(void)printf("<A HREF=\"%s/list/%s\">%s</A><BR>\n",script_name,area,SELECT_LIST);
	(void)printf("<A HREF=\"%s/search/%s\">%s</A><BR>\n",script_name,area,SELECT_SEARCH);
	(void)printf("<A HREF=\"%s/post/%s\">%s</A><BR>\n",script_name,area,SELECT_POST);
	(void)printf("<A HREF=\"%s/delete/%s\">%s</A><BR>\n",script_name,area,SELECT_DELETE);
	(void)printf("</BLOCKQUOTE>\n");
	show_trailer();
}



void choose_area()
{
	char		buffer[8*1024];
	FILE		*src;
	DIR		*dir;
	struct dirent	*dp;
	struct stat	buf;
	int		max,cnt;
	area_t	area[MAX_AREAS];

	if ((dir=opendir("."))) {
		max=0;
		while ((dp=readdir(dir))) {
			if (*dp->d_name!='.') {
				(void)stat(dp->d_name,&buf);
				if (S_ISDIR(buf.st_mode)) {
					(void)sprintf(buffer,"%s/%s",dp->d_name,F_TITLE);
					if ((src=fopen(buffer,"r"))) {
						if (max>=MAX_AREAS)
							show_fatal(ERROR_AREAS);
						(void)fgets(buffer,sizeof(buffer),src);
						area[max].dir=strdup(dp->d_name);
						area[max].title=strdup(strip(buffer));
						max++;
						(void)fclose(src);
					}
					else {
						(void)sprintf(buffer,ERROR_READ_TITLE,dp->d_name);
						show_fatal(buffer);
					}
				}
			}
		}
		(void)closedir(dir);
		if (max)
			qsort((void *)area,(size_t)max,sizeof(area_t),areacmp);
		else
			show_fatal(ERROR_EMPTY);
	}
	else
		show_fatal(ERROR_DIRECTORY);

	show_header(NULL,CHOOSE_HEADER,CHOOSE_DESC);
	(void)printf("<BLOCKQUOTE>\n");
	for (cnt=0;cnt<max;cnt++)
		(void)printf("<A HREF=\"%s/select/%s\">%s</A><BR>\n",script_name,area[cnt].dir,area[cnt].title);
	(void)printf("</BLOCKQUOTE>\n");
	show_trailer();
}



void usage(char *image)
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v]\n",image);
	exit(1);
}


int main(int argc, char *argv[])
{
	int	c;
	char	*ptr;
	char	*area,*command;

	while ((c=getopt(argc,argv,"vh?")) != EOF)
		switch ((char)c) {
		case 'v':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
			exit(0);
		case 'h':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
		case '?':
			usage(argv[0]);
		}

	if (chdir(ROOT))
		show_fatal(ERROR_DIRECTORY);

	(void)putenv("TZ=" TZ);
	script_name=getenv("SCRIPT_NAME");
	ptr=getenv("PATH_INFO");
	if (*ptr=='/')
		ptr++;
	command=ptr;
	if ((area=strchr(command,'/'))) {
		*area++='\0';
		if (chdir(area))
			show_fatal(ERROR_DIRECTORY);
	}

	if (!strcmp(command,"select")) {
		select_area(area);
	}
	else if (!strcmp(command,"list")) {
		list_area(area);
	}
	else if (!strcmp(command,"search")) {
		search_area(area);
	}
	else if (!strcmp(command,"post")) {
		post_area(area);
	}
	else if (!strcmp(command,"delete")) {
		delete_area(area);
	}
	else {
		choose_area();
	}
	return(0);
}
